import Graphics.UI.Yahu

{- PROPSITO DEL EJEMPLO:
				- MOSTRAR UN EJEMPLO DE APLICACIN CON CREACIN Y DESTRUCCIN DINMICA DE OBJETOS
				- MOSTRAR CMO SE APLICA LAYOUT DE MANERA AUTOMATIZADA.
				- ILUSTRAR EL USO DE LOS EVENTOS DE FINALIZACIN

Creamos y destruimos botones (quien dice botones dice cualquier otro objeto, aunque siempre
con la mayour cautela posible), y levamos la cuenta del nmero de botones en pantalla y
del nmero de botones destruidos.
-}

main = yahu init
 where init = do {
	; win <- window [title =: "Crear y destruir botones a placer"]
	--  A cada botn le asignaremos un nmero nico usando este generador de ndices
	; indice <- state [value =: 0]
	{- A poco que se construye una aplicacin dinmica mnimamente compleja, se hace
	necesario guardar referencias a los objetos creados dinmicamente. En este State
	guardamos los botones dinmicos emparejados con su nmero identificador-}
	; botones {- :: State [(Button, Int)] -} <- state [value =: []]
	-- botn para crear botones dinmicos
	; superboton <- button [text =: "crea otro botn"] win
	-- nmero de botones dinmicos actual
	; numbotones <- entry [editable =: False, safeValue =: Just 0] win
	{-nmero de botones dinmicos destruidos. Ms adelante, en un handler nos har falta
	saber si este objeto contina vivo, por lo que especificamos que queremos que se pueda
	saber si el objeto ha sido destruido con "reportDestroy".-}
	; numdestruidos <- entry' [reportDestroy =:. True] [editable =: False, safeValue =: Just 0] win
	{- Uno efecto colateral indeseable del relayout programtico es la variacin arbitraria
	del tamao mnimo de los objetos. Para evitarlo, anotamos un tamao mnimo arbitrario
	de los objetos no dinmicos. Ms adelante lo usaremos para devolverlos a su tamao
	mnimo original-}
	; let canonical_superboton_size = size 140 20
	; let canonical_botones_size = size 140 20
	; let canonical_numbotones_size = size 140 20
	; let canonical_numdestruidos_size = size 140 20
	-- Rutina de relayout programtico.
	; let makelayout = do {
		-- tomamos las referencias a los botones dinmicos
		; indbots <- botones?:value
		-- desechamos los identificadores y nos quedamos con los botones
		; let bots = fst (unzip indbots)
		-- generamos el nuevo layout
		; win!:layout =: lVertical (map expand (superboton:bots)) ^*^
						numdestruidos ^*^ numbotones
		{- En teora, el nuevo layout debera ser perfecto. En la prctica, cuando
		rehacemos el layout tras eliminar un botn, por problemas con la velocidad
		del ordenador, suelen ocurrir dos problemas:
			-no se reacomoda el nuevo layout, quedando un espacio hueco donde estaba
			 el botn eliminado
			-Si el usuario la haba agrandado, la ventana no disminuye de tamao, sino
			 que los elementos toman como tamao mnimo el tamao que tengan en ese
			 momento.
		Ambos problemas se atajan reasignando el tamao mnimo inicial a los elementos:-}
		; superboton!:dim =: canonical_superboton_size
		; numbotones!:dim =: canonical_numbotones_size
		; numdestruidos!:dim =: canonical_numdestruidos_size
		; mapM_ (!: dim =: canonical_botones_size) bots
		}
	-- Rutina de creacin de un nuevo botn
	; let creaboton = do {
		-- extraemos un nuevo nmero identificador para el nuevo botn: num
		; ident <- indice?:value
		; but <- button [text =: "soy el n "++show ident++". Destryeme!"] win
		{- Vamos a hacer que el botn se destruya al pulsar sobre l. Aqu se presenta un
		problema, y es que el evento "action" de los botones tambin se dispara a
		consecuencia de eventos de ratn "left.doubleClick". Se puede dar el caso de
		cliquear sobre el botn, ejecutndose el handler asociado al "action", que lo
		destruye; y antes de que el sistema lo termine de borrar, se haga doble click, con
		lo que vuelve a ejecutar el handler, intentando borrar un objeto ya borrado y
		terminando en error. Para prevenirlo, bloqueamos los eventos de doble click
		izquierdo de ratn, de modo que no se propaguen a niveles superiores y generen
		eventos "action". Para conseguirlo, simplemente retornamos False en el manejador.
		Hay que hacer notar que an as no hemos logrado evitar del todo este problema,
		que sigue presentndose cuando el sistema demora en retirar la imagen grfica del
		botn (por ejemplo si le sorprende en medio de una recoleccin de basura) y el
		usuario puede llegar a producir dos eventos de pulsacin diferenciados, que el
		sistema procesa en dos llamadas al handler de "action".-}
		; but!:on (left $ doubleClick mouse) =: \_->return False
		-- El handler del evento "action"
		; but!:command =: do {
			--destruimos el botn
			; destroy but
			--borramos su referencia en la lista
			; botones!:value <: foldr (\b@(bot,i) bs->if i==ident then bs else b:bs) []
			--decrementamos el nmero de botones mostrado
			; numbotones!:safeValue <: \(Just n)->Just (n-1)
			--rehacemos el layout
			; makelayout
			}
		{- si queremos mostrar el nmero de botones destruidos, basta con que lo
		incrementemos en el handler del evento "action". Pero para mostrar los eventos de
		finalizacin lo haremos as-}
		; but!:on beforeDestroy =: do {
			{-Vamos a modificar el estado del objeto "numdestruidos". Este evento tambin
			se ejecuta cuando finaliza la aplicacin. Si resulta que "numdestruidos" ya
			ha sido eliminado cuando se ejecuta, la aplicacin terminar abrupta e
			incorrectamente. Para evitarlo, slo actualizamos el "entry" si todava
			no ha sido destruido.-}
			; Just destr <- isDestroyed numdestruidos
			; if destr then
				return ()
			  else
				numdestruidos!:safeValue <: \(Just n)->Just (n+1)
			}
		--prximo identificador de botn
		; indice!:value =: ident+1
		--incrementamos el nmero de botones mostrado
		; numbotones!:safeValue <: \(Just n)-> Just (n+1)
		--aadimos el nuevo botn a la lista
		; botones!:value <: ((but,ident):)
		--rehacemos el layout
		; makelayout
		}
	--asignamos el comando de creacin de botones dinmicos
	; superboton!:command =: creaboton
	--hacemos el layout por primera vez
	; makelayout
	}
