import Graphics.UI.Yahu
import LogWindow
import Monad (when, filterM)

{- PROPSITO DEL EJEMPLO:
			- MOSTRAR FORMAS DINMICAS DE USO DE MENS

Este ejemplo esta compuesto casi exclusivamente de mens. Tenemos una barra de mens en la
que el primero es el men maestro, a partir del cual creamos nuevos mens. Aadimos as
mens dinmicamente a la barra de mens, y los populamos con menulabels cuyos comandos nos
permiten aadir a su vez menuitems dinmicamente. A su vez, tenemos un PopupMenu que
populamos dinmicamente con menuChecks para activar/desactivar los mens en la barra
de mens. Por otro lado, mantenemos una lista global de referencias a menuitems generados
dinmicamente, cuyo contenido podemos mostrar a travs de una ventana de logging,
y podemos eliminar las referencias a menuitems destruidos.
-}

{----------------------------------------------------------------------------------------}

--para un men generado dinmicamente, generamos una serie de menuitems
firstelems :: (Int -> MenuItem -> IO ()) -> State Int -> GenericMenu -> IO ()
firstelems registrar ref menu = do {
	--pichando en ste generamos menulabels dinmicamente
	; crealab <- menuLabelAppend [text =: "crea nuevo Menu&Label"] menu
	--pichando en ste generamos menuchecks dinmicamente
	; creachk <- menuLabelAppend [text =: "crea nuevo Menu&Check"] menu
	--pichando en ste generamos submenus dinmicamente, semejantes a su padre
	; creasub <- menuLabelAppend [text =: "crea nuevo Sub&Menu"] menu
	--creamos separadores dinmicamente
	; creasep <- menuLabelAppend [text =: "crea nuevo Menu&Separator"] menu
	--para autodestruir el men
	; elimina <- menuLabelAppend [text =: "&elimina este men!!!"] menu
	--para borrar todos los separadores del men
	; delseps <- menuLabelAppend [text =: "elimi&na los separadores!!!"] menu
	--para habilitar/deshabilitar los menuitems generados dinmicamente de este men
	; enabler <- menuCheckAppend [isChecked =: True, text =: "&Activar/Desactivar dinmicos"] menu
	
	--nmero de menuitems que hemos insertado hasta ahora
	; initnum_menuitems <- menu?:itemCount

	; crealab!:command =: do {
		--pillamos un nuevo identificador para el menuitem
		; i <- ref?:value
		; ref!:value =: i+1
		-- estado de habilitacin inicial
		; activo <- enabler?:isChecked
		--insertamos un nuevo menulabel que, al ser pinchado, se auotdestruir
		; nuevo <- menuLabelAppend [enabled =: activo
			, text =: "&Destryeme!!! ("++show i++")"
			, statusBarHelp =: "Soy un MenuLabel. Si me pinchas me muero. Soy el "++show i++" item"] menu
		--registramos el nuevo menuitem dinmico
		; registrar i (MnILab nuevo)
		; nuevo!:command =: destroy nuevo
		}

	; creachk!:command =: do {
		; i <- ref?:value
		; ref!:value =: i+1
		; activo <- enabler?:isChecked
		--el menucheck podr pincharse varias veces antes de autodestruirse, para mostrar como
		--cambia de estado de chequeo
		; let txt cuenta = "&tictac ("++show i++"): T menos "++show cuenta++" tics"
		; nuevo <- menuCheckAppend [enabled =: activo, text =: txt 3
			, statusBarHelp =: "Soy un MenuCheck. Si me pinchas varias veces me muero. Soy el "++show i++" item"] menu
		; registrar i (MnIChk nuevo)
		--contador de pinchados
		; tmp <- state [value =: 3]
		; nuevo!:command =: do {
			--decrementamos el contador de pinchados
			; hora <- tmp?:value
			; let hora' = hora-1
			; tmp!:value =: hora'
			--si el contador llega a cero, nos autodestruimos; de lo contrario reasignamos el texto
			; if hora'==0 then
				destroy nuevo
			else
				nuevo!:text =: txt hora'
			}
		}

	; creasub!:command =: do {
		; i <- ref?:value
		; ref!:value =: i+1
		; activo <- enabler?:isChecked
		; nuevo <- subMenuAppend [enabled =: activo, text =: "Soy un s&ubmen ("++show i++")"] menu
		; registrar i (MnISubMn nuevo)
		--populamos el submenu con menuitems de la misma forma que estamos haciendo con ste
		; firstelems registrar ref (MnSub nuevo)
		}

	; creasep!:command =: do {
		; i <- ref?:value
		; ref!:value =: i+1
		; nuevo <- menuSeparatorAppend menu
		; registrar i (MnISep nuevo)
		}

	; elimina!:command =: destroy menu

	; delseps!:command =: do {
		; lista <- menuItems menu
		--destruir los separadores es tan sencillo como esto:
		; mapM_ (\mi-> case mi of
			MnISep sep -> destroy sep
			_ -> return ()
			) lista
		}

	; enabler!:command =: do {
		--estado
		; bool <- enabler?:isChecked
		--lista de items de este men
		; lista <- menuItems menu
		-- no cambiamos el estado de los botones definidos en esta funcin, slo los dinmicos!
		; let dinamicos = drop initnum_menuitems lista
		; let cambiar mi = mi!:enabled =: bool
		--por supuesto, no podemos cambiar el estado de los menuSeparator, porque no lo tienen
		; mapM_ (\mi-> case mi of
			MnILab lab -> cambiar lab
			MnIChk chk -> cambiar chk
			MnISubMn sub -> cambiar sub
			MnISep sep -> return ()
			) dinamicos
		}
	}


--principal
main = yahu init
 where init = do {

 	; win <- window [title =: "mens de quita y pon", statusBar =: [VL 1]]

 	--men pop-up
 	; popup <- popupMenu
 	; info <- menuLabelAppend [text =: "A travs de este men se pueden habilitar/deshabilitar los mens dinmicos de la barra de mens"] (MnPopup popup)

 	--contador para enumerar e identificar los mens de la barra de mens generados dinmicamente
 	; ref <- state [value =: 0]
 	--registro de todos los menuitems creados dinmicamente
 	; registro <- state [value =: []]

 	--men maestro de la barra de mens
 	; menuinicial <- inBarMenuAppend [title =: "&men inicial..."] win
 	{-nico menuitem de "menuinicial": aade un nuevo men a la barra de mens. Tambin
 	se dispara pulsando Ctrl-A. No se pueden aadir y eliminar dinmicamente menuitems
 	con combinaciones de teclas de atajo como ste, pues la combinacin de teclas queda
 	irreversiblemente ligada al menuitem-}
 	; appendmenu <- menuLabelAppend [text =: "&Aade un nuevo men...\tCtrl-A"] (MnInBar menuinicial)
 	; appendmenu!:command =: do {
 		--tomamos un nuevo identificador
 		; i <- ref?:value
 		; ref!:value =: i+1
 		--nuevo men
 		; nuevo <- inBarMenuAppend [title =: "&Otro men ("++show i++")..."] win
 		--paralelamente, generamos un menucheck en el men de pop-up, cuya tarea ser
 		--activar/desactivar el men dinmico asociado
 		; enabler <- menuCheckAppend [isChecked =: True
 			, text =: "&activa/desactiva el men "++show i++" en la barra de mens"
 			] (MnPopup popup)
 		; enabler!:command =: (nuevo!:enabled <: not)
 		--cuando el men generado dinmicamente muere, tambin debe morir su enabler
 		; nuevo!:on afterDestroy =: do {
 			; Just muerto <- isDestroyed enabler
 			; when (not muerto) (destroy enabler)
 			}
 		--populamos el men con menuitems para generar dinmicamente otros menuitems
 		--dotamos al men de manera que registre todos los menuitems que cree dinmicamente
 		; firstelems (\ind mi-> registro!:value <: ( (mi, ind) : ) )
 					ref (MnInBar nuevo)
 		}

 	--ventana de logging. Si se quiere ver la implementacin, ver LogWindow.hs
 	; log <- logWindow [dimX =: 600] win

 	--botn cuyo comando muestra en la ventana de logging el estado de los menuitems registrados
 	; showitems <- button [text =: "Mostrar el estado de todos los menuitems registrados"] win
 	; showitems!:command =: do {
 		--borramos el log
 		; logClearMessages log
 		--obtenemos la lista de menuitems registrados
 		; lista <- registro?:value
 		--template que maneja la situacin de menuitem destruido
 		; let siestamuerto mi str otherwise = do {
 			; Just muerto <- isDestroyed mi
 			; if muerto then
 				logMessage log str
 			else
 				otherwise
 			}
 		--escribimos una lnea en el log por cada menuitem
 		; mapM_ (\(mi, ind)-> do {
 			--registramos el identificador
 			; logMessage log ("Num"++show ind++". ")
 			--registramos el estado
 			; case mi of
 				MnISep sep -> siestamuerto sep "Separador destruido\n"
 						(logMessage log "Separador vivo\n")
 				MnILab lab -> siestamuerto lab "MenuLabel destruido\n" (do {
 					; txt <- lab?:text
 					; logMessage log ("MenuLabel vivo: "++txt++"\n")
 					})
 				MnIChk chk -> siestamuerto chk "MenuCheck destruido\n" (do {
 					; txt <- chk?:text
 					; ck <- chk?:isChecked
 					; logMessage log ("MenuCheck vivo: "++txt++", estado "++show ck++"\n")
 					})
 				MnISubMn sub -> siestamuerto sub "SubMenu destruido\n" (do {
 					; txt <- sub?:text
 					; logMessage log ("SubMenu vivo: "++txt++"\n")
 					})
 			})
 			(reverse lista) --volteamos la lista porque estamos insertando por la cabeza
 		}

 	--botn cuyo comando borra del registro todos los menuitems muertos
 	; deregistrar_muertos <- button [text =: "borrar del registro los menuitems destruidos"] win
 	; deregistrar_muertos!:command =:
 		(registro!:value <<: filterM (\(mi,ind)-> do {
 								; Just muerto <- isDestroyed mi
 								; return (not muerto)
 								})
 		)

 	--desplegamos el pop-up al pinchar con el botn derecho en el objeto
 	; let popupevent obj =
 		obj!:on (right mouse) =: \MouseEvent{mousePoint=pos}-> do {
 			; showPopupMenu popup pos obj
 			; return True
 			}
 	; popupevent win
 	; popupevent showitems
 	-- no vale pq no hemos definido LogWindow como MouseListener
 	--; popupevent log
 	; popupevent deregistrar_muertos

 	; win!:layout =: showitems ^#^ log ^#^ deregistrar_muertos
 	}
