import Graphics.UI.Yahu
import Monad(when)
import LogWindow

{- PROPSITO DEL EJEMPLO:
				- MOSTRAR EL USO DEL ScrollCanvas
				- MOSTRAR UN POCO LAS PRIMITIVAS DE DIBUJO

Este programa permite dibujar con el ratn rectngulos permanentes sobre un scrollcanvas.
Se pinta un ractngulo con el ratn pinchando en el scrollcanvas y arrastrando el ratn
hasta darle el tamao deseado.-}

main = yahu init
 where init = do {
	; win <- window [title =: "dibujando rectngulos (y viendo coordenadas)"]
	; let incremento = 10
	; cnv <- scrollCanvas [scrollSize =: size 500 500, dim =: size 150 150] win
	-- * quantum de scrolling en ambos ejes: 10
	; cnv!:pixelsPerScroll =: size incremento incremento
	; borrar <- button [text =: "borrar"] win
	-- Ventana de mensajes de logging. Para ver cmo est hecha, ver el ejemplo LogWindow.hs,
	-- aunque no es necesario para entender este ejemplo.
	; log <- logWindow [dimX =: 400] win
	-- guardamos en una variable la lista de rectngulos definidos. Las coordenadas son
	-- relativas a la superficie virtual del canvas
	; (rectangulos {- ::State [Rect] -} ) <- state [value =: []]
	-- esta otra variable sirve para recordar el punto donde se pinch con el ratn al
	-- empezar a definir un rectngulo
	; initpoint <- state [value =: point 0 0]
	
	{- rutina para repintar el scrollcanvas. Se le pasa una accin que devuelve una lista
	de reas daadas. Dichas reas son relativas a la superficie virtual del scrollcanvas.-}
	; let repintar iodamages = do {
		; damages <- iodamages
		-- tomamos la lista de rectngulos dibujados con el ratn
		; rects <- rectangulos?:value
		{- dibujamos slo los rectngulos que se superponen total o parcialmente con
		las reas daadas. No hay problemas de coherencia pues para los scrollcanvas
		las primitivas de dibujo aceptan coordenadas relativas a su superficie virtual.-}
		; optimizeDraw cnv (mapM_ (\rect-> when (or (map (rectOverlaps rect) damages))
												(drawRectangle cnv rect))
							rects)
		}
	
	-- rutina para repintar el rea visible en pantalla del scrollcanvas
	; let repintarVisible = do {
		-- coordenadas virtuales del rea visible en pantalla del scrollcanvas. El vrtice
		-- superior izquierdo es el valor del atributo scrollPos.
		; visible <- getViewRect cnv
		; repintar (return [visible])
		}
	
	-- evento de repintado del scrollcanvas
	; cnv!:on paint =: repintar
	
	-- evento de pulsacin del ratn en el scrollcanvas. Slo nos interesa el punto donde
	-- se ha pulsado el ratn, que se nos proporciona en coordenadas virtuales del scrollcanvas.
	; cnv!:on (left $ press mouse) =: (\MouseEvent{mousePoint=pt}-> do {
		-- para recordar dnde se puls el ratn
		; initpoint!:value =: pt
		-- posicin del evento de ratn en coordenadas relativas a la representacin del
		-- scrollcanvas en pantalla.
		; scrpt <- calcScrolledPos cnv pt
		-- coordenadas virtuales de la esquina sup. izq. en la pantalla del scrollcanvas.
		; esquina <- cnv?:scrollPos
		-- Dejamos constancia en la ventana de logging de este evento.
		; logMessage log ("pinchado en "++show pt++",\n")
		; logMessage log ("en coords en pantalla "++show scrpt++"\n")
		; logMessage log ("esquina superior izquierda del canvas en "++show esquina++"\n\n")
		-- no nos importa este booleano
		; return True
		})
	
	-- manejador del evento de despulsacin del ratn. en el scrollcanvas. Las coordenadas
	-- son tambin virtuales.
	; cnv!:on (left $ release mouse) =: (\MouseEvent{mousePoint=pt}-> do {
		-- recordamos dnde se puls el ratn
		; initp <- initpoint?:value
		-- el nuevo rectngulo que hay que dibujar se define entre los puntos de pulsacin
		-- y despulsacin del ratn.
		; let rect = rectBetween initp pt
		-- aadimos el rectngulo a la lista
		; rectangulos!:value <: (rect:)
		-- repintamos el rea visible, pues por fuerza el nuevo rectngulo se halla dentro
		-- de dicha rea.
		; repintarVisible
		-- booleano dummy
		; return True
		})
	
	{- evento de pulascin del botn de borrado. Para borrar el scrollcanvas pintaremos
	todos los rectngulos con el color de fondo. Para otros tipos de canvas bastara usar
	la primitiva drawClearAll, pero en los scrollcanvas no funciona correctamente.-}
	; borrar!:command =: do {
		-- le cambiamos el color al pincel, de modo que pinte con el color de fondo. Si
		-- la brocha (drawBrush) no fuera transparente, tambin habra que cambiar su color
		; pen <- cnv?:drawPen
		; bgbr <- cnv?:drawBGBrush
		; cnv!:drawPen =: pen{colorPen=colorBrush bgbr}
		-- coordenadas virtuales del rea en pantalla del scrollcanvas.
		; visible <- getViewRect cnv
		-- repintamos el rea visible, pero como el color del pincel es el del color de
		-- fondo, en la prctica estamos borrando
		; repintarVisible
		-- restituimos el pincel original
		; cnv!:drawPen =: pen
		-- borramos de memoria todos los rectngulos
		; rectangulos!:value =: []
		}
	
	-- botn miscelnea que efecta un scroll programtico sobre el scrollcanvas
	; doScroll <- button [text =: "scroll diagonal"] win
	; let inc = (+incremento)
	; doScroll!:command =: (cnv!:scrollPos <: \(Point x y)-> Point (inc x) (inc y))
	
	; win!:layout =: cnv ^.^ (borrar <.< doScroll) ^.^ log
	}
