import Graphics.UI.Yahu
import System.Directory
import Monad(when)

{- PROPSITO DEL EJEMPLO:
			- MOSTRAR LAS ScrollBar, LOS ListBox, Y LOS EVENTOS paint
			- MOSTRAR LOS DILOGOS PREFABRICADOS DE FICHEROS

Vamos a hacer un visor de imgenes, con dos formas de cargarlas: con un cuadro de dilogo
y mediante una sencilla interfz ad-hoc. El programa quedara mejor con un ScrollCanvas,
pero el propsito del ejemplo son las ScrollBar.-}

-- tipos de archivo soportados, para el cuadro de dilogo
fileTypes = [("BMP", "*.bmp"), ("GIF", "*.gif"), ("JPEG", "*.jpg"), ("All Files", "*.*")]

main = yahu init
 where init = do {
	--------------------------------VISOR--------------------------------------------
	; win <- window [title =: "ImageViewer, Yahu Example"]
	-- nombre del fichero mostrado
	; filename <- entry_ [editable =: False] win
	-- barras de scroll por si la imagen es demasiado grande
	; vscroll <- scrollBar' [orient =:. Vert] [] win
	; hscroll <- scrollBar' [orient =:. Horiz] [] win
	-- canvas sobre el cual pintaremos la imagen
	; canv <- canvas [dim =: size 100 100] win
	; dummy <- bitmap (size 0 0)
	-- Guardamos una referencia a la imagen mostrada en el canvas en una variable
	; image <- state [value =: dummy]
	
	--------------------------------SELECCIN DE FICHEROS---------------------------
	; nb <- notebook [] win

	-- seleccin con cuadro de dilogo
	; nbp1 <- notebookPage [title =: "pick file"] nb
	; openbut <- button [text =: "show Image..."] nbp1
	; defaultButton nbp1 openbut

	--seleccin con lista de directorios y ficheros
	; nbp2 <- notebookPage [title =: "select file"] nb
	; labelFiles <- label [text =: "Files: "] nbp2
	; labelDirs  <- label [text =: "Directories: "] nbp2
	; choiceFiles <- listBox [] nbp2
	; choiceDirs <- listBox [] nbp2
	; spinFiles <- spinButton' [orient =:. Horiz] [spinRange =: (0, 1)] nbp2
	
	--------------------------------FUNCIONES DE AJUSTE------------------------------
	
	-- Rutina de ajuste de las scrollbars al tamao relativo de la imagen y el canvas
	; let adjustScrolls = do {
		; bm <- image?:value
		-- tomamos el tamao de la imagen y ajustamos el rango de las barras
		; (Size maxx maxy) <- bm?:bitmapSize
		; vscroll!:maxRange =: maxy
		; hscroll!:maxRange =: maxx
		-- tomamos el tamao del canvas y ajustamos el tamao de pgina y el thumb
		; (Size cvx cvy) <- canv?:dim
		; vscroll!:pageSize =: cvy
		; vscroll!:thumbSize =: cvy
		; hscroll!:pageSize =: cvx
		; hscroll!:thumbSize =: cvx
		}

	-- Rutina de dibujado de la imagen sobre *una regin* del canvas
	; let drawRect canvas_rect = do {
		; bm <- image?:value
		; posx <- hscroll?:currentPos
		; posy <- vscroll?:currentPos
		; let esq_sup_izq_cnv = rectTopLeft canvas_rect
		; let esq_sup_izq_img = pointAdd (point posx posy) esq_sup_izq_cnv
		; let image_rect = rect esq_sup_izq_img (rectSize canvas_rect)
		; drawCopy bm image_rect canv WrCopy esq_sup_izq_cnv False
		}
	-- Rutina de dibujado de la imagen sobre *todo* el canvas
	; let drawCanvas = do {
		; siz <- canv?:dim
		; drawRect (rect pointZero siz)
		}
	
	-- rutina de carga de imagen en memoria
	; let openImage file = do {
		-- adivinamos el tipo de fichero de imagen a partir de la extensin
		; case bitmapFileTypeFromFileName file of
			Nothing -> return ()
			Just tipo -> do {
				-- intentamos cargar el fichero con el tipo adivinado
				; quiza <- bitmapLoad file tipo
				; case quiza of
					Nothing -> return ()
					Just bm -> do {
						-- actualizamos las variables, liberamos el espacio de la
						-- antigua imagen y dibujamos la nueva imagen
						; filename!:text =: file
						; oldbm <- image?:value
						; destroy oldbm
						; image!:value =: bm
						; adjustScrolls
						; drawClearAll canv
						; drawCanvas
						}
				}
		}
	
	-- Rutina de rellenado de las listas de archivos y directorios a partir de un
	-- directorio raz
	; let fillChoices dir = do {
		-- leemos el contenido del directorio especificado
		; setCurrentDirectory dir
		; currentdir <- getCurrentDirectory
		; paths <- getDirectoryContents currentdir
		-- vaciamos las listas
		; choiceFiles!:strItems =: []
		; choiceDirs!:strItems =: []
		-- rutina que determina la clase (archivo/directorio) de cada entrada del
		-- directorio escaneado
		; let processPath path = do {
			; isdir <- doesDirectoryExist path
			; if isdir then
				-- lo aadimos a la lista de directorios
				strItemAppend choiceDirs path
			else
				-- lo aadimos a la lista de archivos si su extensin es de archivo de imagen
				when (bitmapFileTypeFromFileName path /= Nothing)
					 (strItemAppend choiceFiles path)
			}
		; mapM_ processPath paths
		}
	
	-- rellenamos por primera vez las listas de archivos y directorios
	; fillChoices "."

	
	-----------------------INTERACCIN CON EL USUARIO----------------------------
	
	-- botn para cargar un fichero mediante cuadro de dilogo
	; openbut!:command =: do {
		; quiza <- fileOpenDialog False "select file" "." "" fileTypes 0 win
		; case quiza of
			[] -> return ()
			[file] -> openImage file
		}

	-- cuando se seleccione un item de la lista de directorios, cambiaremos a ste el
	-- directorio actual
	; choiceDirs!:command =: do {
		; ind <- choiceDirs?:selection
		; when (ind/=(-1)) (do {
			; new <- choiceDirs?:strItem ind
			; fillChoices new
			})
		}
	
	-- Cuando se seleccione un item de la lista de archivos, se intentar mostrar por pantalla
	; choiceFiles!:command =: do {
		; ind <- choiceFiles?:selection
		; when (ind/=(-1)) (do {
			; file <- choiceFiles?:strItem ind
			; openImage file
			})
		}
	
	{- Cuando se pulse este spinButton, ciclaremos en el sentido adecuado el item
	actualmente seleccionado en la lista de archivos, ejecutando programticamente
	el handler de evento de cambio de seleccin de dicha lista.-}
	; spinFiles!:on spinPress =: (\haciaabajo-> do {
		; ind <- choiceFiles?:selection
		; when (ind==(-1)) (choiceFiles!:selection =: 0)
		; num <- choiceFiles?:itemCount
		; choiceFiles!:selection <: (if haciaabajo then (+1) else (\x->x-1))
		; acc <- choiceFiles?:command
		; acc
		})
							
	-----------------------------------------------------------------------------------
	
	-- en el evento de repintado reajustamos las barras de scroll, por si ha cambiando
	-- el tamao del canvas, y repintamos las zonas daadas del canvas
	; canv!:on paint =: \io_damage_rects-> do {
		; adjustScrolls
		; damage_rects <- io_damage_rects
		; mapM_ drawRect damage_rects
		}

	-- cada vez que cambia el scroll de una de las barras, repintamos la imagen
	; onAllScrolls (const drawCanvas) vscroll
	; onAllScrolls (const drawCanvas) hscroll
	
	; win!:layout =: (flexMatrix 2 [0] 2 [0] [expand canv, expand vscroll, expand hscroll, voidSpace 0 0]
					^*=^ filename) <*=< insertNB nb [toLayout openbut,
					((labelDirs ^.*^ choiceDirs) ^.^ (labelFiles <.< spinFiles)) ^*^ choiceFiles]
	}

