{- PROPSITO DEL EJEMPLO:
				- MOSTRAR CMO SE PUEDEN CREAR NUEVOS OBJETOS YAHU, SIN TENER QUE ACCEDER A LA INTERFAZ DE BAJO NIVEL

En este ejemplo definimos un nuevo tipo de objeto grfico: los LogWindow, las ventanas
de log. En realidad no son ms que Edit preconfigurados para actuar como logs, y protegidos
de modo que slo se pueda usar la funcionalidad pertinente sobre ellos.
-}

module LogWindow (
	-- * Tipo
	LogWindow
	-- * Funciones generadoras
	, logWindow
	-- * Funciones sobre LogWindow
	, logMessage, logClearMessages, logGetMessages
	) where

-- importamos no slo la funcionalidad estndar de Yahu sino parte de su funcionalidad
-- interna. Para nuestros propsitos, es suficiente.
import Graphics.UI.Yahu.All

-- | Ventana de log
newtype LogWindow = LogWindow Edit

instance Contained LogWindow where
	toLayout (LogWindow ed) = toLayout ed

{- Tal y como est estructurada la biblioteca, es difcil encajar nuevos objetos grficos
que no sean instancia de Widget. Definiendo a tan alto nivel, no tenemos por qu
preocuparnos de para qu sirve toWXWin, sino simplemente suministrarla en base al objeto
subyacente. No obstante, es cierto que a poco que quisiramos hacer cosas "mas raras",
como reunir varios objetos en uno solo, la cosa se complicara bastante.-}
instance Widget LogWindow where
	toWXWin (LogWindow ed) = toWXWin ed

-- Las instancias de Dimensions y Positioned son automticas una vez tenemos la instancia
-- Widget. Tan slo permitiremos que se pueda acceder a estos dos atributos libremente.
instance Dimensions LogWindow
instance Positioned LogWindow

-- | Genera una Logwindow
logWindow :: (Parent w) => [Prop LogWindow] -> w -> IO LogWindow
logWindow props parent = do {
	-- definimos el LogWindow como un Edit con ciertas caractersticas preconfiguradas
	; ed <- edit' [wrapping =:. NoWrap] [editable =: False] parent
	-- construimos el LogWindow
	; let ret = LogWindow ed
	-- ejecutamos las asignaciones de atributos pasadas a la funcin
	; set ret props
	; return ret
	}

-- | Muestra un nuevo mensaje por la ventana de log. Se recomienda que acabe con el carcter de fin de lnea
logMessage :: LogWindow -> String -> IO ()
logMessage (LogWindow ed) msg = do {
	; setInsertionPointEnd ed
	; writeText ed msg
	}

-- | borra los mensajes de la ventana de logs
logClearMessages :: LogWindow -> IO ()
logClearMessages (LogWindow ed) = ed!:text =: ""

-- | Devuelve la concatenacin de todos los mensajes escritos
logGetMessages :: LogWindow -> IO String
logGetMessages (LogWindow ed) = ed?:text

-- | ejemplo de uso de 'LogWindow'
test_main = yahu init
 where init = do {
	; win <- window [title =: "Ejemplo de log"]
	; log <- logWindow [dim =: size 300 200] win
	; inf1 <- label [text =: "Aadir mensaje de log"] win
	; en <- entry_ [] win
	; but <- button [text =: "Borrar los mensajes de log"] win
	; en!:command =: do {
		; msg <- en?:text
		; logMessage log (msg++"\n")
		}
	; but!:command =: logClearMessages log
	; win!:layout =: inf1 ^-^ en ^-^ but ^-^ log
	}


