import Graphics.UI.Yahu
import Monad(when)

{-PROPSITO DEL EJEMPLO:
			- MOSTRAR EL USO DE Alarm
			- MOSTRAR EL USO DE Chronometer
			- MOSTRAR EL USO DE mousePos, idle y finalized
			- MOSTRAR EL USO Y LA UTILIDAD DE LOS OBJETOS INTERRELACIONADOS DE Graphics.UI.Yahu.Join

Este ejemplo muestra una estela de crculos tras el puntero del ratn de color y tamao
configurables, con algunos aadidos para mostrar cmo una aplicacin puede hacer clculos
intensivos (computamos PI del modo ms estpido posible) y a la vez responder fluidamente
al usuario.
-}

-- mediante esta funcin nos aseguramos de la estricticidad de un cmputo en una secuencia
--imperativa para datos no jerrquicos
seqIO :: a -> IO ()
seqIO a = a `seq` return ()

-- | La necesidad de este tipo de datos est en que vamos a mostrar los colores en
--un Entry/SpinButton, por lo que necesitamos datos instancia de Show, Read y Enum
data Colorines = Negro | Gris | Rojo | Verde | Azul | Cyan | Magenta | Amarillo
	deriving (Eq, Show, Read, Enum)

-- convertimos a colores Yahu
fromColorines :: Colorines -> Color
fromColorines c = case c of
	Negro		-> black
	Gris		-> grey
	Rojo		-> red
	Verde		-> green
	Azul		-> blue
	Cyan		-> cyan
	Magenta		-> magenta
	Amarillo	-> yellow

-- escala con nombres para los tamaos
data Escala = Microscopico | MuyChico | Chico | Medianillo | Normalillo | Grande | MuyGrande | Gigantesco
	deriving (Eq, Show, Enum)

fromEscala :: Escala -> Int
fromEscala e = (fromEnum e + 1)*5

--constante de la aplicacin - bien podramos haberla aadido como caracterstica
--configurable, como el color y el tamao
numcirculos = 10

-- el cdigo de inicializacin est organizado en varias secciones
main = yahu init
  where init = do {
  	; win <- window [title =: "demo de Time.hs, App.hs y Join.hs"]
  	
  	------------------------------------------------------------------------------------
  	--SECCIN DE EJEMPLO DE USO DE Graphics.UI.Yahu.Join Y Alarm
  	
  	-- sobre este canvas pintaremos los crculos que siguen al ratn
  	; canv <- canvas [dim =: size 500 400] win
  	-- con este par spin/entry dejaremos que el usuario escoja el color
  	; (spin, showentry) <- spinEntry'
  		[orient =:. Vert, spinWrap =:. True] [spinRange =: (Negro, Amarillo)]
  		[] [dimX =: 50]
  		win
  	-- con este slider etiquetado dejaremos que el usuario escoja el tamao
  	; (slider, [currentval, min, max]) <- sliderFullLabeled'
  		[orient =:. Vert, displayTicks =:. True] [spinRange =: (Microscopico, Gigantesco)]
  		[([], []), ([], []), ([], [])]
  		win
  	-- flag para pintar o no los crculos
  	; running <- checkBox [text =: "running"] win
  	-- instrucciones de uso
  	; instr <- label [text =: "active el checkbox running para que comience el espectaculo"] win
  	-- variable en la que guardamos el estado de dibujo: posicin y tamao de los crculos
  	; puntos <- state [value =: take numcirculos (repeat (point (-100) (-100), fromEscala Microscopico))]
  	-- timer para pintar los frames de la animacin
  	; timer <- alarm [alarmType =: Cyclic, alarmInterval =: 100]
  	
	{- rutina para pintar un frame. toma la lista de (centro,  tamao) de los crculos
	que hay que dibujar y el centro y tamao del crculo ltimo, que hay que borrar-}
  	; let drawPoints points (borrar, sizborrar) = do {
  		-- leemos el color del spin, pero como este est ligado al entry, su valor se
  		--modifica automticamente si escribimos el nombre del color (case sensitive) en
  		--el entry.
  		; c <- spin?:spinValue
  		; let col = fromColorines c
  		-- usamos optimizeDraw porque dibujamos seguidamente varias
  		--funciones y atributos de dibujo
  		; optimizeDraw canv (do {
  			-- tomamos el color de fondo
  			; bgbrush <- canv?:drawBGBrush
  			; let bgcol = colorBrush bgbrush
  			-- ponemos en el pincel el color de fondo para borrar el ltimo crculo
  			; canv!:drawPen <: (\pen->pen{colorPen = bgcol})
  			; drawCircle canv borrar sizborrar
  			-- ponemos en el pincel el color elegido por el usuario para dibujar el resto
  			-- de crculos
  			; canv!:drawPen <: (\pen->pen{colorPen = col})
  			; mapM_ (\(pt, siz) -> drawCircle canv pt siz) points
  			})
  		}

	{- este handler se ejecuta cada vez que el usuario des/marca la casilla. Solo cuando
	est marcada la casilla permitimos que funcione el timer y por tanto la animacin-}
	; running!:command =: do {
		; checked <- running?:isChecked
		; timer!:paused =: not checked
		}
	
	-- este handler se ejecuta peridicamente cuando el timer est activo, y cada vez
	--pinta un frame de la animcacin
	; timer!:command =: do {
		; (Point px py) <- mousePos		--coordenadas absolutas del ratn
		; (Point wx wy) <- win?:pos		--coordenadas de la esquina sup izq de la aplicacin relativas a la pantalla
		; (Point cx cy) <- canv?:pos	--coordenadas de la esquina sup izq del canvas relativas a la aplicacin
		-- coordenadas relativas del ratn
		; let canvpt = point (px - (wx + cx)) (py - (wy - cy))
		-- lista de (centro, tamao) del frame anterior de la animacin
		; points <- puntos?:value
		-- separamos el ltimo elemento, que hay que borrar
		; let (initps, [lastp]) = splitAt (numcirculos-1) points
		-- e insertamos un nuevo elemento, con centro en el cursor del ratn y tamao
		-- especificado por el usuario
		; s <- slider?:spinValue
		; let newpoints = (canvpt, fromEscala s) : initps
		-- dibujamos el nuevo frame
		; drawPoints newpoints lastp
		-- guardamos el frame actual
		; puntos!:value =: newpoints
		}

  	------------------------------------------------------------------------------------
	-- SECCIN DE EJEMPLO DE idle, finalized y Chronometer
	
  	-- los textos explicativos largos es mejor ponerlos en un edit.
  	; info <- edit [dimX =: 200, editable =: False
  		, text =: "sta es una aplicacin muy industriosa. Si se le permite, en sus ratos libres calcula el numero pi con la serie de Leibniz:\n PI = SUM(FROM n=0 TO INF, 4/(n*4+1) - 4/(n*4+3))\n ==>\nPI = 4/1 - 4/3 + 4/5 - 4/7 + 4/9 - 4/11 ...\nAl terminar la aplicacin genera un fichero pi.txt en el que da la aproximacin computada, el nmero de pasos y el tiempo de cmputo."] win
  	-- flag para computar PI en los eventos idle
  	; computarPI <- checkBox [text =: "computar PI"] win
  	; expl_tcomp <- label [text =: "mlseg gastados en computar PI"] win
  	-- visor del tiempo de cmputo de PI
  	; tcomp <- entry [safeValue =: Just 0, editable =: False] win
  	-- estado del cmputo de PI: (valor computado, nmero de pasos)
	; piANDstep <- state [value =: (0.0, 0)]
	-- cronmetro para medir el tiempo que nos pasamos computando PI.
	; crono <- chronometer
	
	-- este handler se ejecuta siempre que la aplicacin est ociosa
	; app!:on idle =: do {
		-- slo computamos un paso si el flag del usuario lo indica as
		; ok <- computarPI?:isChecked
		; when ok (do {
			-- tomamos el ltimo estado computado
			; (pi, n) <- piANDstep?:value
			{- Si es el primer paso, ponemos a cero el cronmetro (como efecto colateral,
			si estaba parado se pone en marcha. Si no, simplemente lo ponemos en marcha-}
			; if n==0 then
				crono!:elapsedTime =: 0
			  else
			    crono!:paused =: False
			; let nPOR4MAS cte = fromIntegral (n*4+cte)
			; let newpi = pi + (4/(nPOR4MAS 1)) - (4/(nPOR4MAS 3))
			{-hay q tener mucho cuidado de que los cmputos sean estrictos cuando es
			necesario, o al final se quedar colgado un buen rato evaluando los cmputos
			suspendidos (hay que ver que lenguaje ms vago)-}
			; seqIO newpi
			-- guardamos el nuevo valor computado
			; piANDstep!:value =: (newpi, n+1)
			-- y paramos el cronmetro
			; crono!:paused =: True
			{- actualizamos el tiempo de cmputo mostrado, *slo* si est desfasado, para
			no sobrecargar a la aplicacin (hay que tener en cuenta que este handler se
			puede llegar a ejecutar muchas veces hasta que vare la cantidad marcada por
			el atributo elapsedTime).-}
			; t <- crono?:elapsedTime
			; Just t' <- tcomp?:safeValue
			; when (t/=t') (tcomp!:safeValue =: Just t)
			})
		-- queremos seguir computando slo si est activada la casilla correspondiente
		; return ok
		}
		
	-- Este handler se ejecuta tras la finalizacin de la aplicacin
	; app!:on finalized =: do {
		; (pi, steps) <- piANDstep?:value
		; tiempo <- crono?:elapsedTime
		-- slo escribimos el archivo si hemos tenido ocasin de computar
		; when (steps > 0) $ 
			writeFile "pi.txt" $
				"valor de PI dando "++show steps++" pasos con la serie de Leibniz: "
				++show pi++"\nTiempo de cmputo en milisegundos: "++show tiempo++"\n"
		}

  	------------------------------------------------------------------------------------

	; win!:layout =: (canv ^*^ (showentry <.< spin <.< instr)) <*=<
		( (slider <.*< ((min ^.#^ currentval) ^.^ max))
		^=^ running ^=^ info ^=^ computarPI ^=^ expl_tcomp ^=^ tcomp )
	
	}
	
	
	