-- para compilar esto har falta el flag -fglasgow-exts
import Graphics.UI.Yahu
import Graphics.UI.Yahu.H98Exts
import Monad(when)

{-PROPSITO DEL EJEMPLO:
			- MOSTRAR EJEMPLOS DE USO DEL MDULO Graphics.UI.Yahu.H98Exts

Un uso de Graphics.UI.Yahu.H98Exts para tareas bsicas se puede ver en el ejemplo
UseExts.hs. Aqu veremos usos ms jugosos (aunque todava triviales): hacemos una
herramienta para consultar atributos independientemente de su tipo (Attr o ConfigAttr).
Tambin mostramos cmo construir una PropsList para la funcin de generacin tratando
uniformente atributos normales y de configuracin.
-}

type Ident = Int

{- estructura de datos para guardar una lista de Attrs. El sistema de tipos requiere que
almacenemos una referencia al objeto con el cual usaremos el atributo, o le demos un
tipo explcito. Para los propsitos del ejemplo, usamos la segunda opcin. En cuanto a
manipular de manera uniforme los valores de los atributos, hay varias posibilidades, pero
la ms fcil es convertirlos de y a String.-}
data AttrsList =
	forall attr prop a. (Props (prop Label) Label, Attrs attr prop, Show a, Read a) =>
	AList Ident (attr Label a) AttrsList
	| ANil

{- estructura de datos para guardar una lista de Attr. La usamos para asignar valores a los
atributos normales paramtricamente.-}
data ListAttr w =
	forall a. (Read a) => ListA Ident (Attr w a) (ListAttr w)
	| NilA

-- devuelve la representacin escrita del valor del atributo con ese identificador, sea
-- normal o de configuracin.
getAttr :: Label ->  AttrsList -> Ident -> IO String
getAttr obj (AList id attr sig) id'
	| id==id' = (obj?::attr) >>= return.show
	| otherwise = getAttr obj sig id'
getAttr obj ANil id' = return ""

-- asigna al atributo con el identificador especificado el valor parseado del String,
-- devolviendo la condicin de xito de la operacin. Evidentemente, slo funciona sobre
-- atributos normales
setAttr :: w -> ListAttr w -> Ident -> String -> IO Bool
setAttr obj (ListA id attr sig) id' strval
	| id==id' = set
	| otherwise = setAttr obj sig id' strval
	where set = case reads strval of
			[(val,_)]-> do {
				; obj!:attr =: val
				; return True
				}
			_ -> return False
setAttr obj NilA id' strval = return False

-- lista de nombres, con identificadores asociados
nombres = ["reportdestroy", "texto", "borde", "justificacion", "tamao", "fuente", "bgcolor", "fgcolor"]
[           reportdestroy ,  texto ,  borde ,  just          ,  tamano ,  fuente ,  bgcol   ,  fgcol   ] = [0..7]

-- pseudolista de atributos genricos
labelAttrs :: AttrsList
labelAttrs = AList reportdestroy reportDestroy $ AList texto text $
			AList borde border $ AList just justify $ AList tamano dim $
			AList fuente font $ AList bgcol bgColor $
			AList fgcol fgColor $ ANil

-- pseudolista de atributos normales
labelNormalAttrs :: ListAttr Label
labelNormalAttrs = ListA texto text $ ListA tamano dim $ ListA fuente font $
					ListA bgcol bgColor $ ListA fgcol fgColor $ NilA

{- pares (identificador_de_atributo, valor_en_String). En este ejemplo la suministramos
a pelo, pero es fcil extender el ejemplo para hacerlo programticamente, de forma que
se pueda especificar cualquier atributo de forma genrica en la inicializacin. Ntese
que asignamos de manera uniforme atributos normales y de configuracin.-}
asignaciones_iniciales :: [(Ident, String)]
asignaciones_iniciales = [(reportdestroy, "True"), (texto, "\"sujeto en pruebas\""),
	(just, "CenterJust"), (bgcol, "rgb(255,128,0)"), (borde, "RaisedB")]


{- El primer argumento son bindings (identificador, valor_string), y el segundo bindings entre
identificadores y atributos. A partir de ambos genera una PropsList, de manera que tratamos
de manera uniforme atributos normales y de configuracin.-}
getPropsList :: [(Ident, String)] -> AttrsList -> PropsList Label
getPropsList ((ident,strval):xs) attrs = (asigAttr attrs) (getPropsList xs attrs)
 where
 	{- esta funcin tiene este tipo porque, al desempaquetar una estructura existencial
 	y empaquetar otra, no podemos dejar escapar las variables cuantificadas, y hay que
 	hacer las dos cosas en la misma funcin. Si no encontramos el identificador o no
 	podemos parsear el String, usamos la funcin "id" para acoplar la salida-}
 	asigAttr :: AttrsList -> PropsList Label -> PropsList Label
	asigAttr (AList ident' attr sig)
		| ident==ident' = case reads strval of
						[(val,_)] -> ((attr =:: val):.)
						_ -> id
		| otherwise = asigAttr sig
	asigAttr ANil = id
getPropsList [] _ = PNil

main = yahu init
 where init = do {
	; win <- window [title =: "getter/setter de atributos normales y de configuracin"]
	-- asignamos en la inicializacin los atributos de esta etiqueta de manera uniforme.
	-- Tambin consultaremos los atributos de la misma forma, sin importar si son normales
	-- o de configuracin
	; sujeto <- label'' (getPropsList asignaciones_iniciales labelAttrs) win
	-- seleccin identificador del atributo sobre el cual ejecutar las acciones getter/setter
	; selec_attrs <- radioBox nombres [text =: "selecciona atributo"] win
	-- a travs de este entry veremos y asignaremos los atributos
	; (interface::Entry ()) <- entry [dimX =: 500] win
	; getter <- button [text =: "getter"] win
	; setter <- button [text =: "setter"] win
	; flag <- checkBox [text =: "indicador de xito al pulsar setter"] win
	-- el usuario no puede manipular el flag
	; onAllMouses (\_->return False) flag
	-- representamos con Show el atributo seleccionado en el Entry
	; getter!:command =: do{
		; ind <- selec_attrs?:selection
		; str <- getAttr sujeto labelAttrs ind
		; interface!:text =: str
		}
	-- asignamos (si es un atributo de configuracin se ignora el comando) a un atributo
	-- el valor parseado (si se puede parsear) del Entry
	; setter!:command =: do {
		; ind <- selec_attrs?:selection
		; str <- interface?:text
		; ok <- setAttr sujeto labelNormalAttrs ind str
		; flag!:isChecked =: ok
		}
	; let margin = marginLeft.marginRight.marginTop.marginBottom.marginWidth 20
	; win!:layout =: (margin sujeto ^*.^ interface) ^.^ selec_attrs ^.^ (getter <.< setter) ^.^ flag
	}

{-COMENTARIO: jugando con los atributos normales, se puede apreciar un comportamiento
poco deseable en la etiqueta: cuando se establece el texto, la fuente u otros, ocurre que
la etiqueta queda demasiado poco ancha o larga. Esto se debe a que el algoritmo que
recalcula el tamao ptimo para el nuevo valor no tiene en cuenta el borde a la hora de
hacer clculos, y suele equivocarse con mltiples reinvocaciones. Este tipo de efectos
espreos son difciles de evitar.-}