import Graphics.UI.Yahu.All
import qualified Graphics.UI.WXH as W
import Graphics.UI.WXH ((.+.))

{- PROPSITO DEL EJEMPLO:
				-MOSTRAR LA FUNCIONALIDAD ESPECIFICA PARA Window Y FloatingWindow

Haremos una ventana de aplicacin que genera dinmicamente nuevas ventanas de aplicacin
con diversas caractersticas configurables, usando parte de la funcionalidad de
Graphics.UI.Yahu.WindowClasses-}

{- | tomamos una ventana, flotante o normal (las flotantes flotan sobre sus padres Window)
y populamos su interior con botones y checkboxes para manipularla.-}
popular_ventana :: (Finalizable w, Container w, ResizableFrame w, HasCaptionButtons w
					, ResizeBorder w, Parent w) => w -> IO ()
popular_ventana win = do {
	--botones para maxi/minimizar
	; min <- button [text =: "minimizar", command =: minimize win] win
	; max <- button [text =: "maximizar", command =: maximize win] win

	--checkbox: si est marcado, se puede arrastrar el borde para cambiar de tamao
	; resize <- checkBox [text =: "borde resizable", isChecked =: True
					, command =: (win!:resizeBorder <: not)] win

	; info <- label [text =: "Slo surte efecto si tiene los botones pertinentes: "] win
	--checkboxes: activacin de los botones de maxi/minimizar en el caption
	; act_min <- checkBox [text =: "Activar botn de minimizar del caption"
					, isChecked =: True, command =: (win!:enableMinimizeButton <: not)] win
	; act_max <- checkBox [text =: "Activar botn de maximizar del caption"
					, isChecked =: True, command =: (win!:enableMaximizeButton <: not)] win
	
	--por si no hay botn de destruccin en el caption
	; destr <- button [text =: "destruir", command =: destroy win] win

	; let sp = voidSpace 0 10
	; win!:layout =: (min <#< max) ^#^ sp ^#^ resize ^#^ sp ^#^ info
					^#^ (act_min <#< act_max) ^#^ sp ^#^ destr
	}
				
-- generamos una lista de atributos de configuracin con la informacin de los checkboxes y el radiobox
configattr :: (StayOnTop w, HasCaptionButtons w) => (Int -> CaptionButtons) -> CheckBox -> CheckBox -> CheckBox -> RadioBox -> IO [ConfigProp w]
configattr tocp stayontop hascaption toolcaption capbutts = do {
	; top <- stayontop?:isChecked
	; hcap <- hascaption?:isChecked
	; tcap <- toolcaption?:isChecked
	; capb <- capbutts?:selection
	; return [stayOnTop =:. top, caption =:. hcap, toolCaption =:. tcap
			, captionButtons =:. tocp capb]
	}

--principal
main = yahu init
 where init = do {
 	; win <- window [title =: "cornucopia de ventanas de aplicacin"]
 	
 	--controles para manipular los atributos de configuracin de las ventanas de
 	--aplicacin que creamos dinmicamente
 	
 	--tipo de objeto: Window o FloatingWindow
 	; tipo <- radioBox ["ventana de aplicacin", "ventana flotante"]
 						[selection =: 0, text =: "tipo de ventana"] win
 	--permanece por encima de todas las dems ventanas de todo el escritorio
 	; stayontop <- checkBox [isChecked =: False, text =: "Atributo de configuracin stayOnTop a True"] win
 	--informacin para el usuario muy larga de escribir en un Label
 	; info <- entry_ [editable =: False, dimX =: 300
 		, text =: "tngase en cuenta que el atributo caption puede ser establecido implcitamente a True por toolCaption y captionButtons"] win
 	--tiene o no tiene caption
 	; hascaption <- checkBox [isChecked =: True, text =: "Atributo de configuracin caption a True"] win
 	--la caption es o no chiquitita (sobreescribe caption=:.True, e interpreta de la misma
 	--forma AllCapButtons y CloseCapButton
 	; toolcaption <- checkBox [isChecked =: False, text =: "Atributo de configuracin toolCaption a True"] win
 	--botones que aparecen en la caption. Si hay algun botn, sobreescribe caption=:.True
 	; capbutts <- radioBox ["NoCapButtons", "CloseCapButton", "AllCapButtons"]
 						[selection =: 2, text =: "Atributo de configuracin captionButtons"] win
	--botn para generar una nueva ventana deacuerdo a las opciones definidas arriba
	; cornucopia <- button [text =: "Nueva ventana!!"] win

 	--el tipo CaptionButtons no es instancia de Enum
 	; let toCapButts int = case int of
 		0 -> NoCapButtons
 		1 -> CloseCapButton
 		2 -> AllCapButtons

 	--generacin de nuevas ventanas
 	; cornucopia!:command =: do {
 		--en funcin del tipo hacemos un Window o un FloatingWindow
 		; tipov <- tipo?:selection
 		; if tipov == 0 then do {
 			--generamos la lista de configuracin
 			; cfg <- configattr toCapButts stayontop hascaption toolcaption capbutts
 			; --generamos la Window
 			; newwin <- window' cfg [resizeBorder =: True
 									, title =: "ventana de aplicacin"]
 			--la populamos con controles
 			; popular_ventana newwin
 		} else do {
 			--generamos la lista de configuracin. Parecera lgico que esta sentencia
 			--fuera antes del IF, pero no se puede debido al sistema de tipos, que se queja
 			--de que no se puede instanciar la variable cfg [ConfigProp Window] y
 			-- [ConfigProp FloatingWindow] simultneamente
 			; cfg <- configattr toCapButts stayontop hascaption toolcaption capbutts
 			; newwin <- floatingWindow' cfg [resizeBorder =: True
 						, title =: "ventana flotante sobre la cornucopia"] win
 			; popular_ventana newwin
 		}}
 	
 	--asignamos layout
 	; let sp = voidSpace 0 10
 	; win!:layout =: tipo ^#^ sp ^#^ stayontop ^#^ sp ^#^ info ^#^ sp ^#^ hascaption ^#^
 					sp ^#^ toolcaption ^#^ sp ^#^ capbutts ^#^ sp ^#^ cornucopia
 	}
 			
 		


